// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition" by Clifford A. Shaffer, Prentice Hall, 2007.
// Source code Copyright (C) 2006 by Clifford A. Shaffer.

// Test for variable nodes implemented using union.
// The test vehicle is an expression tree.
// The program builds and traverses a simple tree.

#include <iostream>
using namespace std;

#include "book.h"

typedef char Operator;
typedef char* Operand;

// Node implementation with a union
enum Nodetype {leaf, intern};  // Enumerate node types

class VarBinNode {       // Generic node class
public:
  Nodetype mytype;       // Stores the type for this node

  union {
    struct {             // Structure for internal node
      VarBinNode* left;  // Left child
      VarBinNode* right; // Right child
      Operator opx;      // Internal node value
    } internal;
    Operand var;         // Leaves just store a value
  };

  VarBinNode(const Operand& val) // Constructor: leaf
    { mytype = leaf; var = val; }

  // Constructor: Internal
  VarBinNode(const Operator& op, VarBinNode* l,
                                 VarBinNode* r) {
    mytype = intern; internal.opx = op;
    internal.left = l; internal.right = r;
  }

  bool isLeaf() { return mytype == leaf; }

  VarBinNode* leftchild() { return internal.left; }
  VarBinNode* rightchild() { return internal.right; }
};

void traverse(VarBinNode* root) { // Preorder traversal
  if (root == NULL) return;
  if (root->isLeaf())
    cout << "Leaf: " << root->var << "\n";
  else {
    cout << "Internal: " << root->internal.opx << "\n";
    traverse(root->leftchild());
    traverse(root->rightchild());
  }
}

int main()
{
  VarBinNode* temp1;
  VarBinNode* temp2;
  VarBinNode* root;
  char* string1 = "Hello1";
  char* string2 = "Another String";

  temp1 = new VarBinNode(string1);
  temp2 = new VarBinNode(string2);
  root = new VarBinNode('+', temp1, temp2);
  traverse(root);
  return(0);
}
